//
//  TechnoVuoRunner.m
//  VuoPluginApp
//
//  Created by Kerry Uchida on 2021 07 12.
//  Copyright © 2021 Kosada. All rights reserved.
//


#import "main.hh"
#import "TechnoVuoRunner.h"
#import <OpenGL/CGLMacro.h>
#include <OpenGL/CGLCurrent.h>
//#import <Vuo/Vuo.h>
//#include <Vuo/VuoRunner.hh>

@interface TechnoVuoRunner ()

@property double center; // The current filter execution center.
@property double amplitude; // The current filter execution center.
@property NSUInteger waveType;
@property (nonatomic) NSImage *sourceImage; // The initial, unfiltered image.
@property (nonatomic) VuoImageFilter *imageFilter; // Used to start, stop, control, and query the composition.

@property (nonatomic) NSMutableDictionary* inPortsDicts;

@end

@implementation TechnoVuoRunner

//@synthesize aWindow;

@synthesize setImageButton;
@synthesize logPortsButton;

@synthesize centerSlider;
@synthesize ampSlider;
@synthesize waveTypePopUp;

@synthesize outputImageView;
@synthesize imageTextField;
@synthesize imageLabel;
@synthesize centerLabel;
@synthesize outputImageLabel;

@synthesize sourceImage;
@synthesize imageFilter;

@synthesize inPortsDicts;

/**
 * Initializes an empty VuoPluginApp document.
 */
- (id)init
{
    self = [super init];
    if (self) {
        self.center = 0;
        self.amplitude = 0;
        self.waveType = 1;
        self.inPortsDicts = [NSMutableDictionary new];
    }
    return self;
}

/**
 * Executes the image filter, and updates the image displayed in the UI.
 */
- (void)updateOutputImage
{
    NSImage *filteredImage = [self.imageFilter filterNSImage:self.sourceImage atTime:self.center];
    [self.outputImageView setImage:filteredImage];
}

/**
 * Loads the specified image.
 */
- (IBAction)setImageInRunningComposition:(NSString *)imagePath {
    self.sourceImage = [[NSImage alloc] initWithContentsOfFile:[imagePath stringByStandardizingPath]];
    if (!self.sourceImage)
    {
        NSAlert *a = [NSAlert new];
        [a setMessageText:@"Unable to read image."];
        [a setInformativeText:imagePath];
       NSWindow *w = [[[self windowControllers] objectAtIndex:0] window];
        [a beginSheetModalForWindow:w completionHandler:^(NSInteger returnCode){}];
        [a release];
        return;
    }

    [self updateOutputImage];
}


- (IBAction)logInputPorts:(id)sender {
    NSArray* inPorts = [self.imageFilter inputPorts];
    NSLog(@"%s inPorts is:%@", __PRETTY_FUNCTION__, inPorts);
    for (id aPort in inPorts) {
        NSString* aPortName = [aPort description];
        NSLog(@"%s aPort is:%@", __PRETTY_FUNCTION__, aPortName);
        NSDictionary* aPortDetails = [self.imageFilter detailsForPort:aPortName];
        NSLog(@"%s aPort details is:%@", __PRETTY_FUNCTION__, aPortDetails);
        [self.inPortsDicts setObject:aPortDetails forKey:aPortName];
    }
    [self testPropertyListFromInput];
    [self setUpPopUpButton];
}

-(void)setUpPopUpButton {
    NSDictionary* aWaveDict = [self.inPortsDicts valueForKey:@"Wave"];
    NSLog(@"%s aWaveDict is:%@", __PRETTY_FUNCTION__, aWaveDict);
    NSArray* allKeys = [aWaveDict allKeys];
    NSLog(@"%s allKeys is:%@", __PRETTY_FUNCTION__, allKeys);
    if ([allKeys containsObject:@"menuItems"]) {
        NSArray* menuItems = [aWaveDict valueForKey:@"menuItems"];
        [self parseMenuItemsAndBuildPopUp:menuItems];
    }
}

-(void)parseMenuItemsAndBuildPopUp:(NSArray*)vuoMenuItems {
    NSMenu* vdmxMenu = [NSMenu new];
    NSLog(@"%s vuoMenuItems is:%@", __PRETTY_FUNCTION__, vuoMenuItems);
    for (NSDictionary* aMenuItem in vuoMenuItems) {
        NSString* aLabel = [aMenuItem valueForKey:@"name"];
        NSString* aValue = [aMenuItem valueForKey:@"value"];
        NSMenuItem* aVDMXMenuItem = [[NSMenuItem alloc] initWithTitle:aLabel
                                                               action:@selector(waveTypeUpdatedFromUI:)
                                                        keyEquivalent:@""];
        NSDictionary* objDict = @{@"Wave" : aValue};
        [aVDMXMenuItem setRepresentedObject:objDict];
        [vdmxMenu addItem:aVDMXMenuItem];
    }
    [self updatePopUpMenu:vdmxMenu];
}

-(void)updatePopUpMenu:(NSMenu*)aNewMenu {
    [self.waveTypePopUp setMenu:aNewMenu];
}

/**
 * Responds to a change in the "wave" pop up made by the user.
 */
- (IBAction)waveTypeUpdatedFromUI:(NSMenuItem*)button {
    NSDictionary *aMenuItemDict = [button representedObject];
    NSLog(@"%s aMenuItemDict is:%@", __PRETTY_FUNCTION__, aMenuItemDict);
    BOOL okUpdate = [self.imageFilter setInputValues:aMenuItemDict];
    NSLog(@"%s okUpdate is:%hhd", __PRETTY_FUNCTION__, okUpdate);
    [self updateOutputImage];
}


/**
 * Responds to a change in the "center" text field value made by the user.
 */
- (IBAction)centerUpdatedFromUI:(id)sender {
    self.center = [sender doubleValue];
    [self updateOutputImage];
}

/**
 * Responds to a change in the "amplitude" slider value made by the user.
 */
- (IBAction)amplitudeUpdatedFromUI:(id)sender {
    self.amplitude = [sender doubleValue];
    [self updateOutputImage];
}


/**
 * Responds to a change in the "image" text field value made by the user.
 */
- (IBAction)imageUpdatedFromUI:(id)sender {
    [self setImageInRunningComposition:[sender stringValue]];
}

/**
 * Responds to a click of the "Set Image" button made by the user.
 */
- (IBAction)setImageButtonClicked:(id)sender {
    [self setImageInRunningComposition:[self.imageTextField stringValue]];
}

/**
 * Sent after the specified window controller loads a nib file if the receiver is the nib file’s owner.
 */
- (void)windowControllerDidLoadNib:(NSWindowController *)aController
{
        // [super windowControllerDidLoadNib:aController];

        // Add any code here that needs to be executed once the windowController has loaded the document's window.

    [self.imageTextField setStringValue:@"/System/Library/Automator/Send Birthday Greetings.action/Contents/Resources/3.jpg"];
    [self setImageButtonClicked:nil];
}

/**
 * Creates and returns a data object that contains the contents of the document, formatted to
 * a specified type.
 */
- (NSData *)dataOfType:(NSString *)typeName error:(NSError **)outError
{
    NSException *exception = [NSException exceptionWithName:@"UnimplementedMethod" reason:[NSString stringWithFormat:@"%@ is unimplemented", NSStringFromSelector(_cmd)] userInfo:nil];
    @throw exception;
    return nil;
}

/**
 * Sets the contents of this document by reading from data of a specified type and returns YES if
 * successful.
 */
- (BOOL)readFromURL:(NSURL *)absoluteURL ofType:(NSString *)typeName error:(NSError **)outError
{
    self.imageFilter = [[VuoImageFilter alloc] initWithComposition:absoluteURL];

    if (!self.imageFilter)
    {
        *outError = [NSError errorWithDomain:NSCocoaErrorDomain code:NSFileReadUnknownError userInfo:nil];
        return NO;
    }

    return YES;
}

/**
 * Tells the delegate that the window is about to close.
 */
- (void)windowWillClose:(NSNotification *)sender
{
        // Stop the running composition associated with this window.
    [self.imageFilter release];
}

/**
 * Returns the name of the document’s sole nib file.
 */
- (NSString *)windowNibName
{
    return @"Document";
}

/**
 * Returns whether the receiver supports autosaving in place.
 */
+ (BOOL)autosavesInPlace
{
    return YES;
}

#pragma mark EXTRA TESTS

-(void)testPropertyListFromInput {
    NSDictionary* propList = [self.imageFilter propertyListFromInputValues];
    NSLog(@"%s propListFromInput is:%@", __PRETTY_FUNCTION__, propList);
}



@end
